from django.contrib.auth import get_user_model
from django.test import Client, TestCase

User = get_user_model()


class BaseTestCase(TestCase):

    def setUp(self):
        super().setUp()
        self.regular_user = User.objects.create_user(
            username="regular",
            email="regular@email.com",
            password="123",
            is_staff=False,
            is_superuser=False,
        )
        self.staff_user = User.objects.create_user(
            username="staff",
            email="staff@email.com",
            password="123",
            is_staff=True,
            is_superuser=False,
        )
        self.superuser = User.objects.create_user(
            username="super",
            email="superuser@gmail.com",
            password="123",
            is_staff=True,
            is_superuser=True,
        )


class TestRequiresAuthenticatedUser(object):

    def test_requires_authenticated_user(self):
        response = self.client.get(self.view_url)
        self.assertEqual(response.status_code, 302)



import datetime
import json

from celery.beat import Service
from django.utils import timezone


def get_periodic_tasks_info():
    from celery import current_app

    schedule = Service(current_app).get_scheduler().get_schedule()
    tasks = []
    for key, entry in schedule.items():
        # entry.is_due() returns (is_due, time_in_seconds_for_next_execution)
        is_due_tpl = entry.is_due()

        next_execution = timezone.now() + datetime.timedelta(seconds=is_due_tpl[1])

        # remove delay between the timezone.now and the schedule entry due date
        next_execution = next_execution.replace(microsecond=0)

        tasks.append(
            {
                "name": key,
                "task": entry.task,
                "args": "(" + ", ".join([json.dumps(arg) for arg in entry.args]) + ")",
                "kwargs": json.dumps(entry.kwargs),
                "is_due": is_due_tpl[0],
                "next_execution": next_execution,
            }
        )

    return tasks



from django.contrib.auth import get_user_model
from django.urls import reverse

from celerybeat_status.tests.utils import BaseTestCase, TestRequiresAuthenticatedUser

User = get_user_model()


class PeriodicTasksStatusListViewTests(TestRequiresAuthenticatedUser, BaseTestCase):
    view_name = "celerybeat_status:periodic-tasks-status"

    def setUp(self):
        super().setUp()
        self.view_url = reverse(self.view_name)

    def test_non_authenticated_user_is_redirected(self):
        response = self.client.get(self.view_url)
        self.assertRedirects(response, "/admin/login/?next=" + self.view_url)

    def test_regular_user_access_returns_forbidden(self):
        self.client.force_login(self.regular_user)
        response = self.client.get(self.view_url)
        self.assertEqual(response.status_code, 403)

    def test_staff_user_access_returns_forbidden(self):
        self.client.force_login(self.staff_user)
        response = self.client.get(self.view_url)
        self.assertEqual(response.status_code, 403)

    def test_superuser_access_returns_success(self):
        self.client.force_login(self.superuser)
        response = self.client.get(self.view_url)
        self.assertEqual(response.status_code, 200)



from django.contrib.auth.mixins import UserPassesTestMixin
from django.utils.translation import gettext_lazy
from django.views.generic.base import TemplateView

from celerybeat_status.helpers import get_periodic_tasks_info


class PeriodicTasksStatusListView(UserPassesTestMixin, TemplateView):
    template_name = "celerybeat_status/periodic_tasks_status_list.html"
    site_url = "/"
    login_url = "admin:login"

    def test_func(self):
        return self.request.user.is_staff and self.request.user.is_superuser

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["title"] = gettext_lazy("Periodic tasks status")
        context["user"] = self.request.user
        context["site_url"] = self.site_url
        context["has_permission"] = self.request.user.is_superuser
        context["tasks"] = get_periodic_tasks_info()

        return context



# -*- coding: utf-8
__title__ = "CeleryBeat Status"
__version__ = "1.0.0"
__author__ = "Vinta Software"
__license__ = "MIT License"
__copyright__ = "Copyright 2017 Vinta Serviços e Soluções Tecnológicas Ltda"

# Version synonym
VERSION = __version__



from django.urls import path

from celerybeat_status.views import PeriodicTasksStatusListView

app_name = "celerybeat_status"

urlpatterns = [
    path(
        "periodic-tasks/",
        PeriodicTasksStatusListView.as_view(),
        name="periodic-tasks-status",
    ),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import shutil
import sys
from io import open

from setuptools import setup

long_description = ""
try:
    with open("README.md") as f:
        long_description = f.read()
except FileNotFoundError:
    print("warning: README.md not found, could not set long_description")


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, "__init__.py")).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [
        dirpath
        for dirpath, dirnames, filenames in os.walk(package)
        if os.path.exists(os.path.join(dirpath, "__init__.py"))
    ]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [
        (dirpath.replace(package + os.sep, "", 1), filenames)
        for dirpath, dirnames, filenames in os.walk(package)
        if not os.path.exists(os.path.join(dirpath, "__init__.py"))
    ]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename) for filename in filenames])
    return {package: filepaths}


version = get_version("celerybeat_status")


if sys.argv[-1] == "publish":
    if os.system("pip freeze | grep twine"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/*")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    shutil.rmtree("dist")
    shutil.rmtree("build")
    shutil.rmtree("django_celerybeat_status.egg-info")
    sys.exit()


setup(
    name="django-celerybeat-status",
    version=version,
    license="MIT",
    description="A simple django admin extension that shows when your periodic are going to run next",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Vinta Software",
    author_email="contact@vinta.com.br",
    packages=get_packages("celerybeat_status"),
    package_data=get_package_data("celerybeat_status"),
    install_requires=[],
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Framework :: Django",
        "Framework :: Django :: 4.2",
        "Framework :: Django :: 5.0",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
)
